"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
/**
 * Throws an internal error because of an incorrect type.
 */
/* istanbul ignore next */
function incorrectNodeType() {
    throw new Error('Internal error: node is not of expected type');
}
/**
 * Assert that the node is an identifier.
 *
 * @param node - The node to assert.
 */
function assertIdentifier(node) {
    /* istanbul ignore if */
    if (!node || !(0, typescript_1.isIdentifier)(node)) {
        incorrectNodeType();
    }
}
/**
 * Assert that the node is a managed initializer.
 *
 * @param node - The node to assert.
 */
function assertManagedInitializer(node) {
    /* istanbul ignore if */
    if (!node ||
        !((0, typescript_1.isAsExpression)(node) || (0, typescript_1.isObjectLiteralExpression)(node) || (0, typescript_1.isParenthesizedExpression)(node))) {
        incorrectNodeType();
    }
}
/**
 * Indicate if the initializer is managed by the transformer.
 *
 * @param node - The node to test.
 * @returns True if this node is a managed expression.
 */
function isManagedInitializer(node) {
    if (node) {
        if ((0, typescript_1.isObjectLiteralExpression)(node)) {
            return true;
        }
        else if ((0, typescript_1.isAsExpression)(node)) {
            return isManagedInitializer(node.expression);
        }
        else if ((0, typescript_1.isParenthesizedExpression)(node)) {
            return isManagedInitializer(node.expression);
        }
    }
    return false;
}
/**
 * The visitor inserting requires to appropriate places.
 */
class RequireInserter {
    constructor(context) {
        this.context = context;
    }
    wants(node) {
        return ((0, typescript_1.isVariableDeclaration)(node) &&
            (0, typescript_1.isIdentifier)(node.name) &&
            isManagedInitializer(node.initializer) &&
            node.name.getText() in this.context.detectedFiles);
    }
    visit(node) {
        const { updateVariableDeclaration } = this.context.factory;
        const identifier = node.name;
        assertIdentifier(identifier);
        return [
            updateVariableDeclaration(node, identifier, node.exclamationToken, node.type, this.updateManagedInitializer(node.initializer, identifier.getText())),
        ];
    }
    /**
     * Update (fill-in) the initializer node. The node is expected to be a managed initializer.
     *
     * @param node - The node to update.
     * @param variable - The name of the variable to update.
     * @returns The updated node.
     */
    updateManagedInitializer(node, variable) {
        assertManagedInitializer(node);
        const { createCallExpression, createIdentifier, createPropertyAccessExpression, createPropertyAssignment, createStringLiteral, updateAsExpression, updateObjectLiteralExpression, updateParenthesizedExpression, } = this.context.factory;
        if ((0, typescript_1.isAsExpression)(node)) {
            return updateAsExpression(node, this.updateManagedInitializer(node.expression, variable), node.type);
        }
        else if ((0, typescript_1.isParenthesizedExpression)(node)) {
            return updateParenthesizedExpression(node, this.updateManagedInitializer(node.expression, variable));
        }
        else {
            return updateObjectLiteralExpression(node, Object.entries(this.context.detectedFiles[variable]).map(([filename, fileDefinition]) => {
                let requireExpression = createCallExpression(createIdentifier('require'), undefined, [
                    createStringLiteral(fileDefinition.filePath),
                ]);
                if (fileDefinition.sourceCode) {
                    requireExpression = createPropertyAccessExpression(requireExpression, createIdentifier('default'));
                }
                return createPropertyAssignment(createStringLiteral(filename), requireExpression);
            }));
        }
    }
}
exports.default = RequireInserter;
//# sourceMappingURL=RequireInserter.js.map