"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const glob_1 = require("glob");
const path_1 = require("path");
/**
 * Throw an error because of configuration problem.
 *
 * @param message - The message for the configuration error.
 * @param item - The index of the item if the message is for an item.
 */
function configurationError(message, item) {
    let itemId = '';
    if (item !== undefined) {
        itemId = ` (item #${item + 1})`;
    }
    throw new Error(`Error in transformer configuration${itemId}: ${message}`);
}
/**
 * Set default values to optional entries in the configuration.
 *
 * @param configurationTarget - The configuration for the target.
 * @returns The configuration with default values.
 */
function setDefault(configurationTarget) {
    const result = { ...configurationTarget };
    result.codeExtensions = result.codeExtensions ?? ['js', 'jsx', 'ts', 'tsx'];
    return result;
}
/**
 * Assert that the provided configuration is correct, from a superficial point of view.
 *
 * @param configuration - The configuration to test.
 */
function assertIsConfiguration(configuration) {
    if (typeof configuration !== 'object' || !configuration) {
        configurationError('configuration must be an object');
    }
    if (!('autoRequires' in configuration)) {
        configurationError('missing “autoRequires” entry');
    }
    if (!Array.isArray(configuration.autoRequires)) {
        configurationError('“autoRequires” must be an array');
    }
}
/**
 * Assert that the configuration item is correct.
 *
 * @param configurationItem - The configuration item.
 * @param index - The item index.
 */
function assertIsConfigurationItem(configurationItem, index) {
    if (typeof configurationItem !== 'object' || !configurationItem) {
        configurationError('configuration must be an object', index);
    }
    if (!('source' in configurationItem)) {
        configurationError('missing “source” entry', index);
    }
    if (typeof configurationItem.source !== 'object') {
        configurationError('“source” entry must be an object', index);
    }
    if (!('glob' in configurationItem.source)) {
        configurationError('missing “source.glob”', index);
    }
    if (typeof configurationItem.source.glob !== 'string') {
        configurationError('“source.glob” must be a string', index);
    }
    if ('ignore' in configurationItem.source &&
        typeof configurationItem.source.ignore !== 'string' &&
        !Array.isArray(configurationItem.source.ignore)) {
        configurationError('“source.ignore” must either be a string or a string array');
    }
    if (!('target' in configurationItem)) {
        configurationError('missing “target” entry', index);
    }
    if (typeof configurationItem.target !== 'object') {
        configurationError('“target” entry must be an object', index);
    }
    if (!('file' in configurationItem.target)) {
        configurationError('missing “target.file”', index);
    }
    if (typeof configurationItem.target.file !== 'string') {
        configurationError('“target.file” must be a string', index);
    }
    if (!('variable' in configurationItem.target)) {
        configurationError('missing “target.variable”', index);
    }
    if (typeof configurationItem.target.variable !== 'string') {
        configurationError('“target.variable” must be a string', index);
    }
    if ('codeExtensions' in configurationItem.target &&
        !Array.isArray(configurationItem.target.codeExtensions)) {
        configurationError('“target.codeExtensions” must be a string array');
    }
}
/**
 * Context for the transformer.
 */
class TContext {
    /**
     * Create the context.
     *
     * @param program - The program object.
     * @param configuration - The provided configuration.
     */
    constructor(program, configuration) {
        /**
         * The detected files.
         */
        this.detectedFiles = {};
        this.basePath = program.getCompilerOptions().rootDir ?? program.getCurrentDirectory();
        // Read the file lists
        const globCache = {
            cache: {},
            statCache: {},
            symlinks: {},
        };
        assertIsConfiguration(configuration);
        this.autoRequires = configuration.autoRequires.map((config, index) => {
            assertIsConfigurationItem(config, index);
            const foundFiles = (0, glob_1.sync)(config.source.glob, {
                ignore: config.source.ignore,
                cwd: this.basePath,
                nomount: true,
                ...globCache,
            })
                .map(filename => ((0, path_1.isAbsolute)(filename) ? (0, path_1.relative)(this.basePath, filename) : filename))
                .filter(filename => !filename.startsWith('..'))
                .map(filename => filename.replace(/\\/g, '/'));
            return { ...setDefault(config.target), foundFiles };
        });
    }
    initNewFile(context) {
        this.currentFactory = context.factory;
        this.detectedFiles = {};
    }
    /**
     * @returns The node factory.
     */
    get factory() {
        return this.currentFactory;
    }
}
exports.default = TContext;
//# sourceMappingURL=TContext.js.map